unit CyDSounder;

interface

uses
  mmsystem, Classes, windows, Math;

type
  TSoundArray  = array [0..0] of Byte;
  PSoundArray  = ^TSoundArray;

  TCyDSounder = class(TThread)
  private
    { Private declarations }
  protected
    procedure Execute; override;
    procedure Process;
  public
  end;


implementation

uses main;

{ TCyDSounder }

procedure WaveOutCheck(res:MMResult);
var
 ErrText:array [0..MAXERRORLENGTH+1] of char;
begin
 if res=MMSYSERR_NOERROR then exit;
 waveOutGetErrorText(res,ErrText,Sizeof(ErrText));
 MessageBox(Form1.Handle,ErrText, 'Error', MB_OK+MB_ICONEXCLAMATION);
end;

procedure TCyDSounder.Execute;
begin
 Process;
end;

procedure TCyDSounder.Process;
const DEFAULT_BLOCKSIZE=64*1024;
var
 BlockSize,waitTime,ii,shift,size:Integer;
 si:TSystemInfo;
 Header:array [0..2] of WAVEHDR;
 hEvent:THANDLE;
 curhdr:WAVEHDR;
 ofs:Int64;
 p:PSoundArray;
begin
 BlockSize:=DEFAULT_BLOCKSIZE;
 waitTime:=round(BlockSize*2000/Form1.wfx.nAvgBytesPerSec);
 GetSystemInfo(si);
 ZeroMemory(@Header,sizeof(Header));
 hEvent:=CreateEvent(nil,false,false,nil);
 try
  WaveOutCheck(waveOutOpen(@Form1.w_hPlay,Form1.w_DeviceID, @Form1.wfx, hEvent,0,CALLBACK_EVENT));
  Header[0].lpData:=VirtualAlloc(nil,round((BlockSize*2+Integer(si.dwPageSize)-1)/si.dwPageSize*si.dwPageSize),MEM_RESERVE+MEM_COMMIT, PAGE_READWRITE);
  if Header[0].lpData=nil then exit;
  Header[1].lpData:=Header[0].lpData+BlockSize;
  Header[0].dwBufferLength:=BlockSize;
  Header[1].dwBufferLength:=BlockSize;
  waveOutCheck(waveOutPrepareHeader(Form1.w_hPlay,@Header[0],sizeof(waveHDR)));
  waveOutCheck(waveOutPrepareHeader(Form1.w_hPlay,@Header[1],sizeof(waveHDR)));
  ii:=0;
  if Form1.DataSize<=BlockSize then ResetEvent(hEvent);
  while Form1.w_PlayPosition<Form1.DataSize do
   begin
    if Form1.w_PlayPosition<Form1.DataSize then
     begin
      Inc(ii);
      curhdr:=Header[ii and 1];
      if BlockSize>Form1.DataSize-Form1.w_PlayPosition then
       BlockSize:=Form1.DataSize-Form1.w_PlayPosition;
      curhdr.dwBufferLength:=BlockSize;
      ofs:=Form1.w_PlayPosition+Form1.DataOffset;
      shift:=(ofs mod si.dwAllocationGranularity);
      ofs:=ofs-shift;
      size:=(BlockSize+ofs+si.dwPageSize-1);
      size:=size-(size mod Integer(si.dwPageSize));
      if size>Form1.wFileSize-ofs then size:=Form1.wFileSize-ofs;
//      ofs:=ofs+shift;
      p:=MapViewOfFile(Form1.w_hData,FILE_MAP_READ, ofs shr 32 and $FFFFFFFF, ofs and $FFFFFFFF, Size);
      try
       CopyMemory(curhdr.lpData, @p[shift], min(size-shift,BlockSize));
      finally
       UnmapViewOfFile(p);
      end;
      waveOutCheck(waveOutWrite(Form1.w_hPlay,@curhdr,sizeof(waveHDR)));
     end;
    if (WAIT_TIMEOUT=WaitForSingleObject(hEvent, WaitTime)) then
     begin
      if not Form1.w_Pause then
       begin
        MessageBox(Form1.Handle,'Wave out error', 'Error', MB_OK+MB_ICONEXCLAMATION);
        break;
       end
      else
       WaitForSingleObject(hEvent,INFINITE);
     end;
    if Form1.w_Stop then break;
    if Form1.w_Restart then
     begin
      //Insert Restart;
     end;
    Form1.w_PlayPosition:=Form1.w_PlayPosition+BlockSize;
   end;
 finally
  waveOutReset(Form1.w_hPlay);
  waveOutUnprepareHeader(Form1.w_hPlay, @Header[0], sizeof(WAVEHDR));
  waveOutUnprepareHeader(Form1.w_hPlay, @Header[1], sizeof(WAVEHDR));
  if (Header[0].lpData<>nil) then VirtualFree(Header[0].lpData,0,Mem_Release);
  waveOutClose(Form1.w_hPlay);
  Form1.w_hPlay:=0;
  CloseHandle(hEvent);
  Free;
 end;
end;

end.
